#! /usr/bin/perl -w

# ResourceIO.pm
# ver 5.2a01;	first version

package ResourceIO;
use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(ReadResFile WriteResFile);

my $program = "$ENV{JTeXEditShellCommands}/ResourceIO";

sub EscapeInput($) {
	my ($input) = @_;
	my ($mark0, $mark) = ('HERE', 'HERE');
	for (my $idx = 0; $input =~ /\n$mark\n/; ++$idx) {
		$mark = $mark0 . $idx;
	}
	return "'$mark'\n$input$mark\n";
}

sub ReadResFile($@) {
	my ($file, @spec) = @_;
	my $in;
	if (@spec) {
		my $out = '';
		for (@spec) {
			my $type = $$_{'type'} or die "spec=>type is not defined.\n";
			die unless (length($type) == 4);
			$type = unpack("H*", $type);
			my $id = $$_{'id'};
			if (defined $id) {
				$out .= "{type=$type, id=$id}\n";
			} else {
				$out .= "{type=$type}\n";
			}
		}
		$out = EscapeInput($out);
		$in = `'$program' -r '$file' <<$out`;
	} else {
		$in = `'$program' '$file'`;
	}
	my %items = ();
	while (length($in) > 0) {
		my ($attr, $name) = (0, '');
		$in =~ s/^{type=([0-9a-fA-F]{8}), id=(-?\d+)// or die "<<$in>>";
		my ($type, $id) = (pack("H*", $1), $2);
		$attr = $1 if ($in =~ s/^, attr=(-?\d+)//);
		$name = pack("H*", $1) if ($in =~ s/^, name=([0-9a-fA-F]*)//);
		$in =~ s/^}\ndata=([0-9a-fA-F]*)\n// or die;
		my $data = pack("H*", $1);
		$items{"$type:$id"} = {'type' => $type, 'id' => $id, 'attr' => $attr, 'name' => $name, 'data' => $data};
	}
	return %items;
}

sub WriteResFile($@) {
	my ($file, @items) = @_;
	my $out = '';
	for my $item (@items) {
		my $type = $$item{'type'} or die;
		die unless (length($type) == 4);
		$type = unpack("H*", $type);
		my $id = $$item{'id'} or die;
		my $attr = $$item{'attr'};
		my $name = $$item{'name'};
		my $data = $$item{'data'};
		$out .= "{type=$type, id=$id";
		$out .= ", attr=$attr" if (defined $attr);
		$out .= ", name=" . unpack("H*", $name) if (defined $name);
		$out .= "}\n";
		if (defined $data) {
			$out .= "data=" . unpack("H*", $data) . "\n";
		} else {
			$out .= "data=-\n";
		}
	}
	$out = EscapeInput($out);
	my $err = `'$program' -w '$file' <<$out`;
	warn "WriteResFile result: $err\n" if $err;
	warn "out was:'$out'\n" if $err;
}
