#! /usr/bin/perl -w
use strict;
use utf8;

#############################################
#
#	file:	SeparateYomi.pl
#		for shell perl
#
#	project:	JTeXEdit
#
#	Copyright (c) 1999-2009 Norihito KOISO
#	All Rights Reserved
#
#############################################
#
#	使用法：
#		"***.idxd" の単語と読みを / で区切って与えれば
#		分解した表を "***.idxd2" に出力する．
#		かなのみは省略される．
#
#############################################
#
#	履歴
#	2011-04-03: use Foundation
#	2009-05-16: UTF-8 tex に対応
#	2008-06-05: perl UTF-8 環境 に対応
#	2003-06-15: 初版
#
#############################################

use Encode;
use Foundation;
# man PerlObjCBridge

sub SetFileTypes($) {
	my ($path) = @_;
	my $sign = NSNumber->numberWithInt_(unpack("N", 'jtxE'));
	my $type = NSNumber->numberWithInt_(unpack("N", 'utf8'));
	my $dict = NSMutableDictionary->dictionary;
	$dict->setObject_forKey_($sign, "NSFileHFSCreatorCode");
	$dict->setObject_forKey_($type, "NSFileHFSTypeCode");
	NSFileManager->defaultManager
		->changeFileAttributes_atPath_($dict, $path);
}

my %Dict = ();
my $DictIsDirty = '';
sub ReadDict(@) {
	my @lines = @_;
	$DictIsDirty = '';
	%Dict = ();
	for my $line (@lines) {
		next if ($line =~ /^%/ or $line eq '');
		my @content = split(/\t/, $line);
		if (@content == 2) {
			$Dict{$content[1]} = $content[0];
		} else {
			warn "辞書の形式が正しくありません:\n$line\n";
		}
	}
}
sub WriteDict() {
	my %buff = ();
	while (my ($text, $yomi) = each %Dict) {
		$buff{"$yomi\t$text"} = 1;
	}
	for my $item (sort keys %buff) {
		print OUT "$item\n";
	}
}

sub OnlyKana($) {
	my ($str) = @_;
	# ASCII or ひらがな or カタカナ
	return $str =~ /^[ -~ァ-ヶー－ぁ-ゖー－]*$/;
}

sub SeparateWords() {
	my %hash = ();
	for my $keyline (keys %Dict) {
		my $valline = $Dict{$keyline};
		my @keys = split('/', $keyline);
		my @vals = split('/', $valline);
		die "'$keyline' => '$valline' の区分の数が合いません．\n"
			unless (@keys == @vals);
		for (my $i = 0; $i <= $#keys; ++$i) {
			my $key = $keys[$i];
			next if OnlyKana($key);
			my $val = $vals[$i];
			my $oldval = $hash{$key};
			if (defined($oldval)) {
				die "異なる読みです．['$key' => '$oldval', '$val']\n"
					unless ($val eq $oldval);
			} else {
				$hash{$key} = $val;
			}
		}
	}
	return %hash;
}

sub ReadFile($\$) {
	my ($path, $enc) = @_;
	open(IN, "<:bytes", $path) or return ();
	my $data = join('', <IN>);
	close(IN);
	return () unless length($data);
	my $text = Encode::decode_utf8($data);
	if (index($text, "\x{FFFD}") < 0) {
		$$enc = ":utf8";
	} else {
		$text = Encode::decode("shift_jis", $data);
		$$enc = ":encoding(shift_jis)";
	}
	return split(/\n|\r/, $text);
}

die ".idxd ファイルを指定して下さい．\n" unless @ARGV;
{
	my $file = shift;
	my $enc;
	my @data = ReadFile($file, $enc) or die "辞書が開けません\n";
	ReadDict(@data);
	%Dict = SeparateWords();
	my $out = $file.'2';
	open(OUT, ">$enc", $out) or die "ファイルが開けません:\n'$out'\n";
	WriteDict();
	close OUT;
	SetFileTypes($out);
}
print "終了．\n";

exit 0;
