#!/usr/bin/perl -w
use strict;
use utf8;
use encoding 'utf8'; # filename

# 2 つのフォルダの中身を再帰的に stat で比較する．
# A) 2 つのフォルダを同時にこのウィンドウにマウスで落とすか，
# 又は
# B.1) "指令:ファイル処理" メニューで "ファイル処理" ダイアログを開き，
# B.2) このウィンドウに 1 つづつフォルダをマウスで落とし，
# B.3) "ファイル処理" ダイアログの "実行" ボタンを押す．

my $FineCompare = 1; # 細かいことを気にする

my ($Obj1, $Obj2) = @ARGV;
die "Error: Pass two objects.\n" unless defined($Obj2);
my $DirMode = -d $Obj1 or -d $Obj2;
print "DiffFileStat '$Obj1' '$Obj2'\n";
my %Table;
my @SameName;
sub AddFiles($);
sub AddFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		closedir(DIR);
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if ($item eq "Icon\r");
			next if (-l "$path/$item");
			AddFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@SameName, $path);
		} else {
			$Table{$1} = $path;
		}
	}
}
my %Sames = ();
sub MakeTable($) {
	my ($path) = @_;
	%Table = ();
	@SameName = ();
	AddFiles($path);
	for my $item (@SameName) {
		$item =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@{$Sames{$1}}, $Table{$1});
			delete $Table{$1};
		}
		push(@{$Sames{$1}}, $item);
	}
}

MakeTable($Obj1) if $DirMode;

my @StatStruct = (
	'',	# dev
	'',	# ino
	$FineCompare ? 'mode' : '',
	$FineCompare ? 'nlink' : '',
	$FineCompare ? 'uid' : '',
	$FineCompare ? 'gid' : '',
	'',	# st_rdev
	'size',
	'',	# atime
	'modify_time',
	'creation_time'
);

sub GetStat($) {
	my ($file) = @_;
	my @stat = stat($file);
	my $result = '';
	for (my $i = 0; $i <= $#StatStruct; ++$i) {
		my ($key, $val) = ($StatStruct[$i], $stat[$i]);
		next if ($key eq '');
		if ($key =~ s/_time//) {
			my @d = localtime($val);
			my $yer = $d[5] + 1900;
			my $mon = $d[4] + 1;
			$val = "$d[5]/$mon/$d[3] $d[2]:$d[1]:$d[0]";
		}
		$result .= "$key=$val; ";
	}
	return $result;
}

sub Compare($$) {
	my ($file1, $file2) = @_;
	my $stat1 = GetStat($file1);
	my $stat2 = GetStat($file2);
	if ($stat1 ne $stat2) {
		print "DIFFER STAT:\n$file1:\n$stat1,\n$file2:\n$stat2\n";
	} else {
		print "same stat:$file1, $file2\n";
	}
}
my @SameResult = ();
my @NotFound = ();
sub ParseFiles($);
sub ParseFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		closedir(DIR);
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if (-l "$path/$item");
			ParseFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			Compare($Table{$1}, $path);
			delete $Table{$1};
		} elsif (defined($Sames{$1})) {
			push(@SameResult, $path);
		} else {
			push(@NotFound, $path);
		}
	}
}
if ($DirMode) { ParseFiles($Obj2); }
else { Compare($Obj1, $Obj2); }
if (@SameResult) {
	print "%%%% multiple files of same name %%%%\n";
	for my $path (@SameResult) {
		print "(M2) $path\n";
		my ($name) = $path =~ m|/([^/]*)$|;
		for my $obj (@{$Sames{$name}}) {
			print "(M1) $obj\n";
		}
	}
}
if (%Table) {
	print "%%%% files (1) not found in (2) %%%%\n";
	for my $key (keys %Table) {
		print "(F1) $Table{$key}\n";
	}
}
if (@NotFound) {
	print "%%%% files (2) not found in (1) %%%%\n";
	for my $path (@NotFound) {
		print "(F2) $path\n";
	}
}