#!/usr/bin/perl -w
use utf8;

# 使用法:
#	2 つのファイルまたはフォルダを比較する．
#	一度に渡せないときは "指令/順序処理..." すること．

use strict;
use encoding 'utf8';

my ($Obj1, $Obj2) = @ARGV;
die "Error: Pass two objects.\n" unless defined($Obj2);
my $DirMode = -d $Obj1 or -d $Obj2;
print "/usr/bin/diff '$Obj1' '$Obj2'\n";
sub NewLine($) {
	my $file = shift;
	open(IN, "<:bytes", $file) or die "$!: '$file'";
	my $text;
	sysread(IN, $text, 1024);
	if ($text =~ /\n/) {
		close IN;
		return $file;
	}
	close IN;
	open(IN, "<:bytes", $file);
	$text = join('', <IN>);
	close IN;
	$text =~ tr/\r/\n/;
	my $tmp = "${file}_00";
	while (-e $tmp) { ++$tmp; }
	open(OUT, ">:bytes", $tmp) or die $!;
	print OUT $text;
	return $tmp;
}
my %Table;
my @SameName;
sub AddFiles($);
sub AddFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		for (@items) { utf8::upgrade($_); }
		closedir(DIR);
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if ($item eq "Icon\r");
			next if (-l "$path/$item");
			AddFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@SameName, $path);
		} else {
			$Table{$1} = $path;
		}
	}
}
my %Sames = ();
sub MakeTable($) {
	my ($path) = @_;
	%Table = ();
	@SameName = ();
	AddFiles($path);
	for my $item (@SameName) {
		$item =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@{$Sames{$1}}, $Table{$1});
			delete $Table{$1};
		}
		push(@{$Sames{$1}}, $item);
	}
}
MakeTable($Obj1) if $DirMode;
sub Compare($$) {
	my ($file1, $file2) = @_;
	print "%%%%%%%%%%%%%%%%\n/usr/bin/diff -a '$file1' '$file2'\n";
	my ($temp1, $temp2) = (NewLine($file1), NewLine($file2));
	my $diff = `/usr/bin/diff -a '$temp1' '$temp2'`;
	unlink $temp1 unless ($temp1 eq $file1);
	unlink $temp2 unless ($temp2 eq $file2);
	print $diff;
}
my @SameResult = ();
my @NotFound = ();
sub ParseFiles($);
sub ParseFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		closedir(DIR);
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if (-l "$path/$item");
			ParseFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			Compare($Table{$1}, $path);
			delete $Table{$1};
		} elsif (defined($Sames{$1})) {
			push(@SameResult, $path);
		} else {
			push(@NotFound, $path);
		}
	}
}
if ($DirMode) { ParseFiles($Obj2); }
else { Compare($Obj1, $Obj2); }
if (@SameResult) {
	print "%%%% multiple files of same name %%%%\n";
	for my $path (@SameResult) {
		print "(M2) $path\n";
		my ($name) = $path =~ m|/([^/]*)$|;
		for my $obj (@{$Sames{$name}}) {
			print "(M1) $obj\n";
		}
	}
}
if (%Table) {
	print "%%%% files (1) not found in (2) %%%%\n";
	for my $key (keys %Table) {
		print "(F1) $Table{$key}\n";
	}
}
if (@NotFound) {
	print "%%%% files (2) not found in (1) %%%%\n";
	for my $path (@NotFound) {
		print "(F2) $path\n";
	}
}