#! /perl -w

# PListIO.pm
# ver 5.2a01;	first version

package PListIO;
use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(PackPerlNumToPList UnpackPListAtomToPerl PerlObjToPListFile PerlObjToPListText PListFileToPerl);

my $PerlAtom_Mark = "\x01";

sub PackPerlNumToPList($) {
	my ($num) = @_;
	return "${PerlAtom_Mark}N$num";
}

sub UnpackPListAtomToPerl($) {
	my ($atom) = @_;
	my ($type, $str) = $atom =~ /^$PerlAtom_Mark(.)(.*$)/;
	if ($type) {
		if ($type eq 'N') {
			$type = ($str =~ /\./) ?  'real' : 'integer';
		} elsif ($type eq 'A') {
			$type = 'data';
		} elsif ($type eq 'E') {
			$type = 'date';
		} else {
			warn "bad argument to UnpackPListAtomToPerl:'$atom'\n";
			$type = 'unknown';
			$str = $atom;
		}
	} else {
		$type = 'string';
		$str = $atom;
	}
	return wantarray ? ($str, $type) : $str;
}

# writer

sub EncodeStrToXML($) {
	my ($str) = @_;
	$str =~ s/\&/\&amp;/g;
	$str =~ s/\</\&lt;/g;
	$str =~ s/\>/\&gt;/g;
	return $str;
}

sub PerlObjToPListText($;$);
sub PerlObjToPListText($;$) {
	my ($ref, $dep) = @_;
	$dep = 0 unless $dep;
	my $type = ref($ref);
	my $out = '';
	my $tabch = "\t";
	my $tabs = $tabch x $dep;
	if ($type eq 'ARRAY') {
		if (@$ref) {
			$out .= "$tabs<array>\n";
			for my $item (@$ref) {
				$out .= PerlObjToPListText($item, $dep + 1);
			}
			$out .= "$tabs</array>\n";
		} else {
			$out .= "$tabs<array/>\n";
		}
	} elsif ($type eq 'HASH') {
		if (%$ref) {
			$out .= "$tabs<dict>\n";
			for my $item (sort keys %$ref) {
				$out .= "$tabs$tabch<key>" . EncodeStrToXML($item) . "</key>\n";
				$out .= PerlObjToPListText($$ref{$item}, $dep + 1);
			}
			$out .= "$tabs</dict>\n";
		} else {
			$out .= "$tabs<dict/>\n";
		}
	} else {
		$out .= $tabs;
		$ref = EncodeStrToXML($ref);
		my ($mark, $str) = $ref =~ /^$PerlAtom_Mark(.)(.*)$/;
		if ($mark) {
			if ($mark eq 'N') {
				if ($str =~ /\./) { $out .= "<real>$str</real>"; }
				else { $out .= "<integer>$str</integer>"; }
			} elsif ($mark eq 'B') {
				$out .= ($str ? "<true/>" : "<false/>");
			} elsif ($mark eq 'E') {
				$out .= "<date>$str</date>";
			} elsif ($mark eq 'A') {
				$out .= "<data>$str</data>";
			}
		} else {
			$out .= "<string>$ref</string>";
		}
		$out .= "\n";
	}
	return $out;
}

sub PerlObjToPListFile($$) {
	my ($path, $obj) = @_;
	open(OUT, ">:utf8", $path) or die "Can't open '$path' $!";
	my $header =<<END;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
END
	my $footer = "</plist>\n";
	print OUT $header;
	print OUT PerlObjToPListText($obj);
	print OUT $footer;
	close(OUT);
}

# reader

sub DecodeXMLtoStr($) {
	my ($str) = @_;
	$str =~ s/\&amp;/&/g;
	$str =~ s/\&lt;/</g;
	$str =~ s/\&gt;/>/g;
	return $str;
}

sub XMLTextToPerlObj($);

sub XMLArraytoPerlList($) {
	my ($text) = @_;
	my @list = ();
	while (length($text)) {
		my ($pre, $key, $item);
		($pre, $key, $text) = $text =~ m|^(.*?)<(.*?)>(.*)$|s or die;
		return (\@list, $text) if ($key eq '/array');
		($item, $text) = XMLTextToPerlObj("<$key>$text") or die;
		push(@list, $item);
	}
	die "/array not found";
}

sub XMLDictToPerlHash($) {
	my ($text) = @_;
	my %hash = ();
	while (length($text)) {
		my ($pre, $key, $item);
		($pre, $key, $text) = $text =~ m|^(.*?)<(.*?)>(.*)$|s or die;
		return (\%hash, $text) if ($key eq '/dict');
		die unless ($key eq 'key');
		($key, $text) = $text =~ m|^(.*?)</key>(.*)$|s or die;
		($item, $text) = XMLTextToPerlObj($text) or die;
		$hash{$key} = $item;
	}
	die "/dictionary not found";
}

sub XMLTextToPerlObj($) {
	my ($text) = @_;
	my ($pre, $key, $one);
	($pre, $key, $text) = $text =~ m|^(.*?)<(.*?)>(.*)$|s or die;
	($key, $one) = $key =~ m|^(\w+)(/?)$| or die;
	if ($key eq 'array') {
		return ([], $text) if ($one);
		return XMLArraytoPerlList($text);
	} elsif ($key eq 'dict') {
		return ({}, $text) if ($one);
		return XMLDictToPerlHash($text);
	} elsif ($key eq 'string') {
		return ('', $text) if ($one);
		($pre, $text) = $text =~ m|^(.*?)</string>(.*)$|s or die;
		return (DecodeXMLtoStr($pre), $text);
	} elsif ($key eq 'real' or $key eq 'integer' or $key eq 'date' or $key eq 'data') {
		($pre, $text) = $text =~ m|^(.*?)</$key>(.*)$|s or die;
		my $mark = ($key eq 'real' or $key eq 'integer') ? 'N' : ($key eq 'date') ? 'E' : 'A';
		return ("$PerlAtom_Mark$mark" . DecodeXMLtoStr($pre), $text); 
	} elsif ($key eq 'true' or $key eq 'false') {
		die "Error at XMLTextToPerlObj" unless ($one);
		return ($key eq 'true' ? "${PerlAtom_Mark}B1" : "${PerlAtom_Mark}B", $text);
	} else {
		die "Error at XMLTextToPerlObj";
	}
}

sub PListFileToPerl($) {
	my ($file) = @_;
	open(IN, "<:utf8", $file) or die "Can't open '$file'";
	my $text = join('', <IN>);
	close(IN);
	$text =~ m|<plist version.*?>(.*)$|s or die "<plist version*> not found.";
	my ($plist, $post) = XMLTextToPerlObj($1);
	die "</plist> not found." unless ($post =~ m|^[\s\n]*</plist>|);
	return $plist;
}

1;
