#! perl -w

use strict;

BEGIN {
	my ($dir) = $0 =~ m|^(.*)/[^/]*$|;
	unshift(@INC, $dir);
}
use ExpandTeX;

# by N. KOISO

# 2006/04/23: shell perl (OS X)
# 2000/11/20: first version (MacJPerl)

my $Usage =<<'EOU';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

usage (in japanese) 使用法
詳しい説明は ExpandTeX.pm を参照．
ExpandTeX.pl, ExpandTeX.pm, ExpandTeX.CSs を同じフォルダにおき，
下の説明に従ったテキスト ABC.tex をこのウインドウ (ExpandTeX.pl) にドラッグする．
ただし，ABC.xtra の定義が必要な場合もある．
展開結果は ABC.xpnd に出力される．

ABC.tex:
	TeX 形式のテキスト．ExpandSample.tex を参照のこと．
	1) 単純なマクロを
		'%% BEGIN_DEFINE_MACROS' 行と
		'%% END_DEFINE_MACROS' 行で囲む．
	単純なマクロとは以下のようなものをさす．
	"\def\cs#1,#2 {..}" などは許されない．
		\let\name\other
		\let\name=\other
		\def\name{other}
		\def\name#1#2#3{#1 and #2 and #3}
		\newcommand{\name}{other}
		\renewcommand{\name}{other}
		\newcommand{\name}[2]{#1 and #2}
		\renewcommand{\name}[2]{#1 and #2}
	2) 実際に展開すべき部分を
		'%% BEGIN_EXPAND_MACROS' 行と
		'%% END_EXPAND_MACROS' 行で囲む．

ABC.xtra:
		複雑なマクロを Perl で記述する．
		また，text に定義があるが展開したくないものを宣言する．
		ExpandSample.xtra を参照のこと．
		SetMacro($cs, &sub)
			$cs が現れたときに &sub を実行する．
			sub($name, $in)
				$name は $cs. 通常不使用．
				$in は続くテキストへのリファレンス．
				置換の結果を返すこと．
				sub は ExpandTeX:: 内で実行される．
				$cs の引数は GetArgs($$in, $num) で受け取れる．
		なお，xtra は実際の展開の直前に eval され，
		重複する単純なマクロは無効になる．
		

ExpandTeX.CSs:
	標準の展開しないマクロの引数の個数を定義する．
	'numArgs = \d' 行で以下のマクロの引数の個数を \d と定義する．
	その他の各行は 1 個のマクロ '\name' を記述する．

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
EOU

my $EndExpand = "%% END_EXPAND_MACROS\n";
my %Mark = (
	"%% BEGIN_DEFINE_MACROS\n", "%% END_DEFINE_MACROS\n",
	"%% BEGIN_EXPAND_MACROS\n", $EndExpand
);

my $File = shift
or die $Usage;
my $CSs = $0;
$CSs =~ s|/[^/]*$|/ExpandTeX.CSs|;

sub FileToNumArgs($);
sub ExpandFile($);

print "\n**** ExpandTEX ****\n";
FileToNumArgs($CSs);
print "$File\n";
ExpandFile($File);
print "\n**** end ****\n";
exit(0);

## subs

sub FileToNumArgs($) {
	my ($file) = @_;
	open IN, $file or die $Usage;
	my @lines = split(/[\n\r]/, join('', <IN>));
	close IN;
	my $num = 0;
	for (@lines) {
		if (/^numArgs\s*=\s*(\d)/) { $num = $1; }
		elsif (/^(\\.\S*)/) { ExpandTeX::SetNumArgs($num, $1); }
	}
}

sub ExpandFile($) {
	my ($file) = @_;
	my $xtra = $file;
	$xtra =~ s|(/[^/]+)\.[^/.]*$|$1.xtra| or $xtra .= '.xtra';
	my $xtradone = '';
	my $xpnd = $file;
	$xpnd =~ s|(/[^/]+)\.[^/.]*$|$1.xpnd.tex| or $xpnd .= '.xpnd.tex';
	open IN, $file or die $Usage;
	my @lines = split(/[\r\n]/, join('', <IN>));
	@lines = map("$_\n", @lines);
	close IN;
	open OUT, ">$xpnd" or die "Can't write $xpnd\n$!\n";
	while (@lines) {
		$_ = shift @lines;
		print OUT;
		next unless $Mark{$_};
		my $end = $Mark{$_};
		my $in = '';
		while (@lines) {
			$_ = shift @lines;
			goto job if ($_ eq $end);
			$in .= $_;
		}
		die "\nNot found: $end";
job:
		if ($end eq $EndExpand) {
			unless ($xtradone) {
				if (open(IN2, $xtra)) {
					my $code = join("", <IN2>);
					$code =~ tr/\r/\n/;
					ExpandTeX::Eval($code);
				}
				$xtradone = 1;
			}
			my $out = ExpandTeX::Expand($in);
			print OUT $out;
		} else {
			print OUT $in;
			ExpandTeX::Define($in);
		}
		print OUT $end;
	}
	close OUT;
	#MacPerl::SetFileInfo('jtxE', 'TEXT', $xpnd);
	my @err = ExpandTeX::Undefined();
	return unless @err;
	print "\n*** There are unknown macros in '$file'. ****";
	print "\n*** Rewrite 'ExpandTeX.CSs' or '$xtra'. ****\n";
	print join("\n", @err), "\n\n";
}
