#! /usr/bin/perl -w
use strict;
use utf8;
use encoding 'utf8';

# 入力形式: '.pfb'
# 出力形式:
my $Format = '.otf';
# 出力形式は .ttf を選ぶこともできる．

print "\nFontForge.pl\n";

my $WorkDir = "$ENV{'HOME'}/Desktop/FontForgeFonts";
unless (-e $WorkDir) {
	mkdir($WorkDir) or die "作業用フォルダが作れません: $WorkDir";
}
sub GetOutDir($) {
	my ($type) = @_;
	$type = substr($type, 1) if ($type =~ /^\./);
	my $dir = "$WorkDir/$type";
	unless (-e $dir) {
		mkdir($dir) or die "作業用フォルダが作れません: $dir";
	}
	return $dir;
}

sub TeXCommand($) {
	my ($cmd) = @_;
	my $path = $ENV{'JTeXEditTeXBinPath'};
	return $path ? "$path/$cmd" : $cmd;
}

my $InputDir = '';
sub FindFile($) {
	my ($name) = @_;
	my ($type) = $name =~ /\.([^.]*)$/ or die "File without types";
	return "$InputDir/$type/$name" if ($InputDir and -e "$InputDir/$type/$name");
	return "$WorkDir/$type/$name" if ($type eq 'tfm' and -e "$WorkDir/$type/$name");
	my $command = TeXCommand('kpsewhich');
	my $path = `$command $name`;
	chomp $path;
	return $path;
}

my $ScriptPath = '';
unless ($ScriptPath) {
	my $script =<<EOS;
#!/usr/local/bin/fontforge
format = "$Format"
i = 1
while (i < \$argc)
	Open(\$argv[i])
	if (\$argv[i+1] != "")
		MergeKern(\$argv[i+1]);
	endif
	if (\$order == 2 && format == ".otf")
		SetFontOrder(3)
		SelectAll()
		Simplify(128+32+8, 1.5)
		ScaleToEm(1000)
	elseif (\$order == 3 && format == ".ttf")
		ScaleToEm(2048)
		RoundToInt()
	endif
	Generate(\$argv[i+2])
	i = i + 3
endloop
EOS
	$ScriptPath = "$WorkDir/FontForgeScript.pe";
	open(OUT, ">$ScriptPath") or die;
	print OUT $script;
	close(OUT);
}

my (@Success, @Failed, @NoOption);

sub ParseFont($) {
	my ($src) = @_;
	print "\n$src\n";
	my ($dir, $body, $type) = $src =~ m|^(.*)/([^/]*)\.([^./]*)$|;
	if ($type ne 'pfb') { print "> .pfb ではないので無視します\n"; return; }
	my $tfm = FindFile("$body.tfm");
	unless ($tfm) {
		print "> .tfm がありません．.mf から作ります．\n";
		my $trash = GetOutDir("_trash_");
		my $tfmdir = GetOutDir("tfm");
		if ($InputDir and -e "$InputDir/mf") {
			print "> > chdir '$InputDir/mf'\n";
			chdir("$InputDir/mf");
		}
		print "> > mktextfm --destdir '$trash' $body\n";
		my $command = TeXCommand('mktextfm');
		`$command --destdir '$trash' $body`;
		if (-e "$trash/$body.tfm") {
			print "> > rename($trash/$body.tfm, $tfmdir/$body.tfm)\n";
			rename("$trash/$body.tfm", "$tfmdir/$body.tfm");
			print "> > $body.tfm を作りました\n";
		} else {
			print "> > 失敗: $body.tfm が作れませんでした\n";
		}
	}
	my $opt = FindFile("$body.tfm") or FindFile("$body.afm");
	if ($opt) {
		print "> 合成で .otf を作ります: $opt\n";
	} else {
		print "> .afm, .tfm がありません．なしで .otf を作ります\n";
	}
	my $trg = GetOutDir($type) . "/$body$Format";
	my $shell = "fontforge '$ScriptPath' '$src' '$opt' '$trg'";
	print "> $shell\n";
	my $result = `$shell`;
	print "Result: $result\n" if ($result);
	if (-e $trg) {
		print "> 成功\n";
		if ($opt) { push(@Success, $body); }
		else { push(@NoOption, $body); }
	} elsif (not $opt) {
		print "> 失敗\n";
		push(@Failed, $body);
	} else {
		print "> 失敗したので .afm, .tfm なしで .otf を作ります\n";
		$shell = "fontforge '$ScriptPath' '$src' '' '$trg'";
		print "> $shell\n";
		$result = `$shell`;
		print "Result: $result\n" if ($result);
		if (-e $trg) {
			print "> > 成功\n";
			push(@NoOption, $body);
		} else {
			print "> > 失敗\n";
			push(@Failed, $body);
		}
	}
}

# main
{
	($InputDir) = $ARGV[0] =~ m|^(.*)/pfb/[^/]*$|;
	for (sort @ARGV) { ParseFont($_); }
	print "\n\n#### 結果一覧 ####\n\n";
	print "失敗:\n\t", join("\n\t", @Failed), "\n\n" if (@Failed);
	print "成功 (.tfm, .afm なし):\n\t", join("\n\t", @NoOption), "\n\n" if (@NoOption);
	print "成功:\n\t", join("\n\t", @Success), "\n\n" if (@Success);
}
